@@ -33,16 +33,24 @@
 #include "seahorse-marshal.h"
 #include "seahorse-servers.h"
 #include "seahorse-transfer-operation.h"
+#include "seahorse-unknown.h"
 #include "seahorse-unknown-source.h"
-#include "seahorse-unknown-key.h"
 #include "seahorse-util.h"
 
+#include "common/seahorse-bind.h"
 #include "common/seahorse-registry.h"
 
 #ifdef WITH_PGP
 #include "pgp/seahorse-server-source.h"
 #endif
 
+/**
+ * SECTION:seahorse-context
+ * @short_description: This is where all the action in a Seahorse process comes together.
+ * @include:libseahorse/seahorse-context.h
+ *
+ **/
+
 /* The application main context */
 SeahorseContext* app_context = NULL;
 
@@ -50,6 +58,7 @@ enum {
     ADDED,
     REMOVED,
     CHANGED,
+    REFRESHING,
     LAST_SIGNAL
 };
 
@@ -74,6 +83,7 @@ struct _SeahorseContextPrivate {
     GHashTable *objects_by_source;          /* See explanation above */
     GHashTable *objects_by_type;            /* See explanation above */
     guint notify_id;                        /* Notify for GConf watch */
+    SeahorseMultiOperation *refresh_ops;    /* Operations for refreshes going on */
     SeahorseServiceDiscovery *discovery;    /* Adds sources from DNS-SD */
 };
 
@@ -103,7 +113,10 @@ seahorse_context_class_init (SeahorseContextClass *klass)
                 NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, SEAHORSE_TYPE_OBJECT);    
     signals[CHANGED] = g_signal_new ("changed", SEAHORSE_TYPE_CONTEXT, 
                 G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET (SeahorseContextClass, changed),
-                NULL, NULL, seahorse_marshal_VOID__OBJECT_UINT, G_TYPE_NONE, 2, SEAHORSE_TYPE_OBJECT, G_TYPE_UINT);    
+                NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, SEAHORSE_TYPE_OBJECT);    
+    signals[REFRESHING] = g_signal_new ("refreshing", SEAHORSE_TYPE_CONTEXT, 
+                G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET (SeahorseContextClass, refreshing),
+                NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, SEAHORSE_TYPE_OPERATION);    
 }
 
 /* init context, private vars, set prefs, connect signals */
@@ -125,13 +138,12 @@ seahorse_context_init (SeahorseContext *sctx)
     
     /* The context is explicitly destroyed */
     g_object_ref (sctx);
-    
 }
 
 static void
-hash_to_slist (gpointer object, gpointer value, gpointer user_data)
+hash_to_ref_slist (gpointer object, gpointer value, gpointer user_data)
 {
-	*((GSList**)user_data) = g_slist_prepend (*((GSList**)user_data), value);
+	*((GSList**)user_data) = g_slist_prepend (*((GSList**)user_data), g_object_ref (value));
 }
 
 /* release all references */
@@ -145,9 +157,11 @@ seahorse_context_dispose (GObject *gobject)
     
     /* Release all the objects */
     objects = NULL;
-    g_hash_table_foreach (sctx->pv->objects_by_source, hash_to_slist, &objects);
-    for (l = objects; l; l = g_slist_next (l)) 
-	    seahorse_context_remove_object (sctx, l->data);
+    g_hash_table_foreach (sctx->pv->objects_by_source, hash_to_ref_slist, &objects);
+    for (l = objects; l; l = g_slist_next (l)) {
+            seahorse_context_remove_object (sctx, l->data);
+            g_object_unref (G_OBJECT (l->data));
+    }
     g_slist_free (objects);
 
     /* Gconf notification */
@@ -171,6 +185,10 @@ seahorse_context_dispose (GObject *gobject)
     g_slist_free (sctx->pv->sources);
     sctx->pv->sources = NULL;
     
+    if (sctx->pv->refresh_ops)
+	    g_object_unref (sctx->pv->refresh_ops);
+    sctx->pv->refresh_ops = NULL;
+    
     G_OBJECT_CLASS (seahorse_context_parent_class)->dispose (gobject);
 }
 
@@ -190,11 +208,17 @@ seahorse_context_finalize (GObject *gobject)
     g_assert (sctx->pv->sources == NULL);
     g_assert (sctx->pv->auto_sources == NULL);
     g_assert (sctx->pv->discovery == NULL);
+    g_assert (sctx->pv->refresh_ops == NULL);
     g_free (sctx->pv);
     
     G_OBJECT_CLASS (seahorse_context_parent_class)->finalize (gobject);
 }
 
+/**
+* seahorse_context_for_app:
+*
+* Returns: the application main context as #SeahorseContext
+*/
 SeahorseContext*
 seahorse_context_for_app (void)
 {
@@ -227,7 +251,8 @@ seahorse_context_new (guint flags)
 			g_object_unref (app_context);
         
 		g_object_ref (sctx);
-		gtk_object_sink (GTK_OBJECT (sctx));
+		g_object_ref_sink (sctx);
+		g_object_unref (sctx);
 		app_context = sctx;
         
 		refresh_keyservers (NULL, 0, NULL, sctx);
@@ -253,33 +278,41 @@ seahorse_context_destroy (SeahorseContext *sctx)
         app_context = NULL;
 }
 
+static gboolean                
+take_source (SeahorseContext *sctx, SeahorseSource *sksrc)
+{
+	g_return_val_if_fail (SEAHORSE_IS_SOURCE (sksrc), FALSE);
+	if (!g_slist_find (sctx->pv->sources, sksrc)) {
+		sctx->pv->sources = g_slist_append (sctx->pv->sources, sksrc);
+		return TRUE;
+	}
+	
+	return FALSE;
+}
+
 void                
 seahorse_context_take_source (SeahorseContext *sctx, SeahorseSource *sksrc)
 {
-    g_return_if_fail (SEAHORSE_IS_SOURCE (sksrc));
+	g_return_if_fail (SEAHORSE_IS_SOURCE (sksrc));
     
-    if (!sctx)
-        sctx = seahorse_context_for_app ();
-    g_return_if_fail (SEAHORSE_IS_CONTEXT (sctx));
+	if (!sctx)
+		sctx = seahorse_context_for_app ();
+	g_return_if_fail (SEAHORSE_IS_CONTEXT (sctx));
 
-    if (!g_slist_find (sctx->pv->sources, sksrc))
-        sctx->pv->sources = g_slist_append (sctx->pv->sources, sksrc);
+	take_source (sctx, sksrc);
 }
 
 void
 seahorse_context_add_source (SeahorseContext *sctx, SeahorseSource *sksrc)
 {
-    g_return_if_fail (SEAHORSE_IS_SOURCE (sksrc));
+	g_return_if_fail (SEAHORSE_IS_SOURCE (sksrc));
+    
+	if (!sctx)
+		sctx = seahorse_context_for_app ();
+	g_return_if_fail (SEAHORSE_IS_CONTEXT (sctx));
 
-    if (!sctx)
-        sctx = seahorse_context_for_app ();
-    g_return_if_fail (SEAHORSE_IS_CONTEXT (sctx));
-    
-    if (g_slist_find (sctx->pv->sources, sksrc))
-        return;
-    
-    sctx->pv->sources = g_slist_append (sctx->pv->sources, sksrc);
-    g_object_ref (sksrc);
+	if (take_source (sctx, sksrc))
+		g_object_ref (sksrc);
 }
     
 void
@@ -320,8 +353,8 @@ seahorse_context_find_source (SeahorseContext *sctx, GQuark ktype,
     for (l = sctx->pv->sources; l; l = g_slist_next (l)) {
         ks = SEAHORSE_SOURCE (l->data);
         
-        if (ktype != SKEY_UNKNOWN && 
-            seahorse_source_get_ktype (ks) != ktype)
+        if (ktype != SEAHORSE_TAG_INVALID && 
+            seahorse_source_get_tag (ks) != ktype)
             continue;
         
         if (location != SEAHORSE_LOCATION_INVALID && 
@@ -332,7 +365,7 @@ seahorse_context_find_source (SeahorseContext *sctx, GQuark ktype,
     }
     
     /* If we don't have an unknown source for this type, create it */
-    if (location == SEAHORSE_LOCATION_MISSING && location != SKEY_UNKNOWN) {
+    if (location == SEAHORSE_LOCATION_MISSING && location != SEAHORSE_TAG_INVALID) {
         ks = SEAHORSE_SOURCE (seahorse_unknown_source_new (ktype));
         seahorse_context_add_source (sctx, ks);
         return ks;
@@ -341,6 +374,16 @@ seahorse_context_find_source (SeahorseContext *sctx, GQuark ktype,
     return NULL;
 }
 
+
+/**
+* seahorse_context_find_sources:
+* @sctx: the context to work with
+* @ktype: the type of the key to match. Or SEAHORSE_TAG_INVALID
+* @location: the location to match. Or SEAHORSE_LOCATION_INVALID
+*
+* Returns: A list of seahorse sources matching @ktype and @location as #GSList. Must
+*  be freed with #g_slist_free
+*/
 GSList*
 seahorse_context_find_sources (SeahorseContext *sctx, GQuark ktype,
                                SeahorseLocation location)
@@ -356,8 +399,8 @@ seahorse_context_find_sources (SeahorseContext *sctx, GQuark ktype,
     for (l = sctx->pv->sources; l; l = g_slist_next (l)) {
         ks = SEAHORSE_SOURCE (l->data);
         
-        if (ktype != SKEY_UNKNOWN && 
-            seahorse_source_get_ktype (ks) != ktype)
+        if (ktype != SEAHORSE_TAG_INVALID && 
+            seahorse_source_get_tag (ks) != ktype)
             continue;
         
         if (location != SEAHORSE_LOCATION_INVALID && 
@@ -418,16 +461,9 @@ seahorse_context_remote_source (SeahorseContext *sctx, const gchar *uri)
 
 
 static void
-object_changed (SeahorseObject *sobj, SeahorseKeyChange change, SeahorseContext *sctx)
+object_notify (SeahorseObject *sobj, GParamSpec *spec, SeahorseContext *sctx)
 {
-    g_signal_emit (sctx, signals[CHANGED], 0, sobj, change);
-}
-
-static void
-object_destroyed (SeahorseObject *sobj, SeahorseContext *sctx)
-{
-    /* When objects are destroyed elsewhere */
-    seahorse_context_remove_object (sctx, sobj);
+	g_signal_emit (sctx, signals[CHANGED], 0, sobj);
 }
 
 static gpointer                 
@@ -518,8 +554,7 @@ seahorse_context_take_object (SeahorseContext *sctx, SeahorseObject *sobj)
         sctx = seahorse_context_for_app ();
     g_return_if_fail (SEAHORSE_IS_CONTEXT (sctx));
     g_return_if_fail (SEAHORSE_IS_OBJECT (sobj));
-    g_return_if_fail (sobj->_id != 0);
-    g_return_if_fail (!sobj->attached_to);
+    g_return_if_fail (seahorse_object_get_id (sobj) != 0);
     
     ks = hashkey_by_source (seahorse_object_get_source (sobj), 
                             seahorse_object_get_id (sobj));
@@ -528,14 +563,13 @@ seahorse_context_take_object (SeahorseContext *sctx, SeahorseObject *sobj)
 
     g_object_ref (sobj);
 
-    sobj->attached_to = sctx;
+    g_object_set (sobj, "context", sctx, NULL);
     g_hash_table_replace (sctx->pv->objects_by_source, ks, sobj);
     setup_objects_by_type (sctx, sobj, TRUE);
     g_signal_emit (sctx, signals[ADDED], 0, sobj);
     g_object_unref (sobj);
     
-    g_signal_connect (sobj, "changed", G_CALLBACK (object_changed), sctx);
-    g_signal_connect (sobj, "destroy", G_CALLBACK (object_destroyed), sctx);
+    g_signal_connect (sobj, "notify", G_CALLBACK (object_notify), sctx);
 }
 
 guint
@@ -563,47 +597,93 @@ seahorse_context_get_object (SeahorseContext *sctx, SeahorseSource *sksrc,
 }
 
 typedef struct _ObjectMatcher {
-    
-    SeahorseObjectPredicate *kp;
-    gboolean many;
-    GList *objects;
-    
+	SeahorseObjectPredicate *kp;
+	gboolean many;
+	SeahorseObjectFunc func;
+	gpointer user_data;
 } ObjectMatcher;
 
 gboolean
 find_matching_objects (gpointer key, SeahorseObject *sobj, ObjectMatcher *km)
 {
-    if (km->kp && seahorse_object_predicate_match (km->kp, SEAHORSE_OBJECT (sobj)))
-        km->objects = g_list_prepend (km->objects, sobj);
+	gboolean matched;
+	
+	if (km->kp && seahorse_object_predicate_match (km->kp, SEAHORSE_OBJECT (sobj))) {
+		matched = TRUE;
+		(km->func) (sobj, km->user_data);
+	}
 
-    /* Terminate search */
-    if (!km->many && km->objects)
-        return TRUE;
+	/* Terminate search */
+	if (!km->many && matched)
+		return TRUE;
 
-    /* Keep going */
-    return FALSE;
+	/* Keep going */
+	return FALSE;
 }
 
-GList*             
-seahorse_context_get_objects (SeahorseContext *sctx, SeahorseSource *sksrc)
+static void
+add_object_to_list (SeahorseObject *object, gpointer user_data)
 {
-    SeahorseObjectPredicate kp;
-    ObjectMatcher km;
+	GList** list = (GList**)user_data;
+	*list = g_list_prepend (*list, object);
+}
 
-    if (!sctx)
-        sctx = seahorse_context_for_app ();
-    g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
-    g_return_val_if_fail (sksrc == NULL || SEAHORSE_IS_SOURCE (sksrc), NULL);
+GList*
+seahorse_context_find_objects_full (SeahorseContext *self, SeahorseObjectPredicate *pred)
+{
+	GList *list = NULL;
+	ObjectMatcher km;
 
-    memset (&kp, 0, sizeof (kp));
-    memset (&km, 0, sizeof (km));
-    
-    km.kp = &kp;
-    km.many = TRUE;
-    kp.source = sksrc;
-    
-    g_hash_table_find (sctx->pv->objects_by_source, (GHRFunc)find_matching_objects, &km);
-    return km.objects;
+	if (!self)
+		self = seahorse_context_for_app ();
+	g_return_val_if_fail (SEAHORSE_IS_CONTEXT (self), NULL);
+	g_return_val_if_fail (pred, NULL);
+
+	memset (&km, 0, sizeof (km));
+	km.kp = pred;
+	km.many = TRUE;
+	km.func = add_object_to_list;
+	km.user_data = &list;
+
+	g_hash_table_find (self->pv->objects_by_source, (GHRFunc)find_matching_objects, &km);
+	return list; 
+}
+
+void
+seahorse_context_for_objects_full (SeahorseContext *self, SeahorseObjectPredicate *pred,
+                                   SeahorseObjectFunc func, gpointer user_data)
+{
+	ObjectMatcher km;
+
+	if (!self)
+		self = seahorse_context_for_app ();
+	g_return_if_fail (SEAHORSE_IS_CONTEXT (self));
+	g_return_if_fail (pred);
+	g_return_if_fail (func);
+
+	memset (&km, 0, sizeof (km));
+	km.kp = pred;
+	km.many = TRUE;
+	km.func = func;
+	km.user_data = user_data;
+
+	g_hash_table_find (self->pv->objects_by_source, (GHRFunc)find_matching_objects, &km);
+}
+
+GList*
+seahorse_context_get_objects (SeahorseContext *self, SeahorseSource *source)
+{
+	SeahorseObjectPredicate pred;
+
+	if (!self)
+		self = seahorse_context_for_app ();
+	g_return_val_if_fail (SEAHORSE_IS_CONTEXT (self), NULL);
+	g_return_val_if_fail (source == NULL || SEAHORSE_IS_SOURCE (source), NULL);
+
+	seahorse_object_predicate_clear (&pred);
+	pred.source = source;
+	
+	return seahorse_context_find_objects_full (self, &pred);
 }
 
 SeahorseObject*        
@@ -650,28 +730,24 @@ seahorse_context_find_objects (SeahorseContext *sctx, GQuark ktype,
     return seahorse_context_find_objects_full (sctx, &pred);
 }
 
-GList*
-seahorse_context_find_objects_full (SeahorseContext *sctx, SeahorseObjectPredicate *skpred)
+static void
+verify_each_object (gpointer key, gpointer value, gpointer user_data)
 {
-    ObjectMatcher km;
+	gpointer k;
 
-    if (!sctx)
-        sctx = seahorse_context_for_app ();
-    g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
-
-    memset (&km, 0, sizeof (km));
-    
-    km.kp = skpred;
-    km.many = TRUE;
-    
-    g_hash_table_find (sctx->pv->objects_by_source, (GHRFunc)find_matching_objects, &km);
-    return km.objects; 
+	g_assert (SEAHORSE_IS_OBJECT (value));
+	k = hashkey_by_source (seahorse_object_get_source (value), 
+	                       seahorse_object_get_id (value));
+	g_assert (k == key);
 }
 
-gboolean
-seahorse_context_owns_object (SeahorseContext *sctx, SeahorseObject *sobj)
+void
+seahorse_context_verify_objects (SeahorseContext *self)
 {
-    return sobj->attached_to == sctx;
+	if (!self)
+		self = seahorse_context_for_app ();
+	g_return_if_fail (SEAHORSE_IS_CONTEXT (self));
+	g_hash_table_foreach (self->pv->objects_by_source, verify_each_object, self);
 }
 
 void 
@@ -683,17 +759,17 @@ seahorse_context_remove_object (SeahorseContext *sctx, SeahorseObject *sobj)
         sctx = seahorse_context_for_app ();
     g_return_if_fail (SEAHORSE_IS_CONTEXT (sctx));
     g_return_if_fail (SEAHORSE_IS_OBJECT (sobj));
+    g_return_if_fail (seahorse_object_get_id (sobj) != 0);
     
     k = hashkey_by_source (seahorse_object_get_source (sobj), 
                            seahorse_object_get_id (sobj));
     
     if (g_hash_table_lookup (sctx->pv->objects_by_source, k)) {
-        g_return_if_fail (sobj->attached_to == sctx);
+        g_return_if_fail (seahorse_object_get_context (sobj) == sctx);
 
         g_object_ref (sobj);
-        g_signal_handlers_disconnect_by_func (sobj, object_changed, sctx);
-        g_signal_handlers_disconnect_by_func (sobj, object_destroyed, sctx);
-        sobj->attached_to = NULL;
+        g_signal_handlers_disconnect_by_func (sobj, object_notify, sctx);
+        g_object_set (sobj, "context", NULL, NULL);
         g_hash_table_remove (sctx->pv->objects_by_source, k);
         setup_objects_by_type (sctx, sobj, FALSE);
         g_signal_emit (sctx, signals[REMOVED], 0, sobj);    
@@ -707,12 +783,14 @@ seahorse_context_remove_object (SeahorseContext *sctx, SeahorseObject *sobj)
  */
 
 /**
- * seahorse_context_get_default_key
+ * seahorse_context_get_default_key:
  * @sctx: Current #SeahorseContext
- * 
- * Returns: the secret key that's the default key 
+ *
+ * Returns: the secret key that's the default key
+ *
+ * Deprecated: No replacement
  */
-SeahorseKey*
+SeahorseObject*
 seahorse_context_get_default_key (SeahorseContext *sctx)
 {
     SeahorseObject *sobj = NULL;
@@ -732,18 +810,18 @@ seahorse_context_get_default_key (SeahorseContext *sctx)
     
     g_free (id);
     
-    if (SEAHORSE_IS_KEY (sobj))
-	    return SEAHORSE_KEY (sobj);
-    return NULL;
+    return sobj;
 }
 
 /**
- * seahorse_context_get_discovery
+ * seahorse_context_get_discovery:
  * @sctx: #SeahorseContext object
- * 
+ *
  * Gets the Service Discovery object for this context.
  *
- * Return: The Service Discovery object. 
+ * Returns: The Service Discovery object.
+ *
+ * Deprecated: No replacement
  */
 SeahorseServiceDiscovery*
 seahorse_context_get_discovery (SeahorseContext *sctx)
@@ -756,52 +834,34 @@ seahorse_context_get_discovery (SeahorseContext *sctx)
     return sctx->pv->discovery;
 }
 
-SeahorseOperation*  
-seahorse_context_refresh_local (SeahorseContext *sctx)
-{
-    SeahorseSource *ks;
-    SeahorseMultiOperation *mop = NULL;
-    SeahorseOperation *op = NULL;
-    GSList *l;
-    
-    if (!sctx)
-        sctx = seahorse_context_for_app ();
-    g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
-
-    for (l = sctx->pv->sources; l; l = g_slist_next (l)) {
-        ks = SEAHORSE_SOURCE (l->data);
-        
-        if (seahorse_source_get_location (ks) == SEAHORSE_LOCATION_LOCAL) {
-            if (mop == NULL && op != NULL) {
-                mop = seahorse_multi_operation_new ();
-                seahorse_multi_operation_take (mop, op);
-            }
-            
-            op = seahorse_source_load (ks, 0);
-            
-            if (mop != NULL)
-                seahorse_multi_operation_take (mop, op);
-        }
-    }   
-
-    return mop ? SEAHORSE_OPERATION (mop) : op;  
-}
-
-static gboolean 
-refresh_local (SeahorseContext *sctx)
-{
-    SeahorseOperation *op = seahorse_context_refresh_local (sctx);
-    g_return_val_if_fail (op != NULL, FALSE);
-    g_object_unref (op);
-    return FALSE;
-}
-
 void
-seahorse_context_refresh_local_async (SeahorseContext *sctx)
+seahorse_context_refresh_auto (SeahorseContext *sctx)
 {
-    g_idle_add_full (G_PRIORITY_LOW, (GSourceFunc)refresh_local, sctx, NULL);
-}
+	SeahorseSource *ks;
+	SeahorseOperation *op = NULL;
+	GSList *l;
+    
+	if (!sctx)
+		sctx = seahorse_context_for_app ();
+	g_return_if_fail (SEAHORSE_IS_CONTEXT (sctx));
+	
+	if (!sctx->pv->refresh_ops)
+		sctx->pv->refresh_ops = seahorse_multi_operation_new ();
 
+	for (l = sctx->pv->sources; l; l = g_slist_next (l)) {
+		ks = SEAHORSE_SOURCE (l->data);
+        
+		if (seahorse_source_get_location (ks) == SEAHORSE_LOCATION_LOCAL) {
+
+			op = seahorse_source_load (ks);
+			g_return_if_fail (op);
+			seahorse_multi_operation_take (sctx->pv->refresh_ops, op);
+		}
+		
+	}
+	
+	g_signal_emit (sctx, signals[REFRESHING], 0, sctx->pv->refresh_ops);
+}
 
 SeahorseOperation*  
 seahorse_context_search_remote (SeahorseContext *sctx, const gchar *search)
@@ -956,7 +1016,7 @@ seahorse_context_transfer_objects (SeahorseContext *sctx, GList *objects,
         /* Export from this key source */
         from = seahorse_object_get_source (sobj);
         g_return_val_if_fail (from != NULL, FALSE);
-        ktype = seahorse_source_get_ktype (from);
+        ktype = seahorse_source_get_tag (from);
         
         /* Find a local keysource to import to */
         if (!to) {
@@ -969,7 +1029,7 @@ seahorse_context_transfer_objects (SeahorseContext *sctx, GList *objects,
         }
         
         /* Make sure it's the same type */
-        if (ktype != seahorse_source_get_ktype (to)) {
+        if (ktype != seahorse_source_get_tag (to)) {
             /* TODO: How can we warn caller about this. Do we need to? */
             g_warning ("destination is not of type: %s", 
                        g_quark_to_string (ktype));
@@ -1062,6 +1122,7 @@ GList*
 seahorse_context_discover_objects (SeahorseContext *sctx, GQuark ktype, 
                                    GSList *rawids)
 {
+    SeahorseOperation *op = NULL;
     GList *robjects = NULL;
     GQuark id = 0;
     GSList *todiscover = NULL;
@@ -1069,7 +1130,6 @@ seahorse_context_discover_objects (SeahorseContext *sctx, GQuark ktype,
     SeahorseSource *sksrc;
     SeahorseObject* sobj;
     SeahorseLocation loc;
-    SeahorseOperation *op;
     GSList *l;
 
     if (!sctx)
@@ -1079,7 +1139,7 @@ seahorse_context_discover_objects (SeahorseContext *sctx, GQuark ktype,
     /* Check all the ids */
     for (l = rawids; l; l = g_slist_next (l)) {
         
-        id = seahorse_source_canonize_id (ktype, (gchar*)l->data);
+        id = seahorse_context_canonize_id (ktype, (gchar*)l->data);
         if (!id) {
             /* TODO: Try and match this partial id */
             g_warning ("invalid id: %s", (gchar*)l->data);
@@ -1127,79 +1187,64 @@ seahorse_context_discover_objects (SeahorseContext *sctx, GQuark ktype,
         /* Running operations ref themselves */
         g_object_unref (op);
     }
-    
+
     /* Start a discover process on all todiscover */
     if (seahorse_gconf_get_boolean (AUTORETRIEVE_KEY) && todiscover) {
         op = seahorse_context_retrieve_objects (sctx, ktype, todiscover, NULL);
-
-        /* Add unknown objects for all these */
-        sksrc = seahorse_context_find_source (sctx, ktype, SEAHORSE_LOCATION_MISSING);
-        for (l = todiscover; l; l = g_slist_next (l)) {
-            if (sksrc) {
-                sobj = seahorse_unknown_source_add_object (SEAHORSE_UNKNOWN_SOURCE (sksrc), 
-                                                           GPOINTER_TO_UINT (l->data), op);
-                robjects = g_list_prepend (robjects, sobj);
-            }
-        }
-        
-        g_slist_free (todiscover);
         
         /* Running operations ref themselves */
         g_object_unref (op);
     }
-    
+
+    /* Add unknown objects for all these */
+    sksrc = seahorse_context_find_source (sctx, ktype, SEAHORSE_LOCATION_MISSING);
+    for (l = todiscover; l; l = g_slist_next (l)) {
+        if (sksrc) {
+            sobj = seahorse_unknown_source_add_object (SEAHORSE_UNKNOWN_SOURCE (sksrc), 
+                                                       GPOINTER_TO_UINT (l->data), op);
+            robjects = g_list_prepend (robjects, sobj);
+        }
+    }
+
+    g_slist_free (todiscover);
+
     return robjects;
 }
 
 SeahorseObject*
-seahorse_context_object_from_dbus (SeahorseContext *sctx, const gchar *key, guint *uid)
+seahorse_context_object_from_dbus (SeahorseContext *sctx, const gchar *key)
 {
     SeahorseObject *sobj;
-    const gchar *t = NULL;
-    gchar *x, *alloc = NULL;
-    
-    /* Find the second colon, if any */
-    t = strchr (key, ':');
-    if (t != NULL) {
-        t = strchr (t + 1, ':');
-        if(t != NULL) {
-            key = alloc = g_strndup (key, t - key);
-            ++t;
-        }
-    }
     
     /* This will always get the most preferred key */
     sobj = seahorse_context_find_object (sctx, g_quark_from_string (key), 
                                          SEAHORSE_LOCATION_INVALID);
     
-    if (uid)
-        *uid = 0;
-        
-    /* Parse out the uid */
-    if (SEAHORSE_IS_KEY (sobj) && sobj && t) {
-        glong l = strtol (t, &x, 10);
-            
-        /* Make sure it's valid */
-        if (*x || l < 0 || l >= seahorse_key_get_num_names (SEAHORSE_KEY (sobj)))
-            sobj = NULL;
-        else if (uid)
-            *uid = (guint)l;
-    }
-    
     return sobj;
 }
 
 gchar*
-seahorse_context_object_to_dbus (SeahorseContext *sctx, SeahorseObject *sobj, guint uid)
+seahorse_context_object_to_dbus (SeahorseContext *sctx, SeahorseObject *sobj)
 {
-    return seahorse_context_id_to_dbus (sctx, seahorse_object_get_id (sobj), uid);
+    return seahorse_context_id_to_dbus (sctx, seahorse_object_get_id (sobj));
 }
 
 gchar*
-seahorse_context_id_to_dbus (SeahorseContext* sctx, GQuark id, guint uid)
+seahorse_context_id_to_dbus (SeahorseContext* sctx, GQuark id)
 {
-    if (uid == 0)
-        return g_strdup (g_quark_to_string (id));
-    else
-        return g_strdup_printf ("%s:%d", g_quark_to_string (id), uid);
+	return g_strdup (g_quark_to_string (id));
+}
+
+GQuark
+seahorse_context_canonize_id (GQuark ktype, const gchar *id)
+{
+	SeahorseCanonizeFunc canonize;
+
+	g_return_val_if_fail (id != NULL, 0);
+    
+	canonize = seahorse_registry_lookup_function (NULL, "canonize", g_quark_to_string (ktype), NULL);
+	if (!canonize) 
+		return 0;
+	
+	return (canonize) (id);
 }
